@@ -40,7 +40,7 @@ enum assignment_fsm_event {
 	ASSIGNMENT_EV_CONN_RELEASING,
 };
 
-void assignment_fsm_init(void);
+void bssap_extend_osmux(struct msgb *msg, uint8_t cid);
 
 int reassignment_request_to_lchan(enum assign_for assign_for, struct gsm_lchan *lchan, struct gsm_lchan *to_lchan,
 				  int tsc_set, int tsc);
@@ -50,3 +50,4 @@ int reassignment_request_to_chan_type(enum assign_for assign_for, struct gsm_lch
 void assignment_fsm_start(struct gsm_subscriber_connection *conn, struct gsm_bts *bts,
 			  struct assignment_request *req);
 void assignment_reset(struct gsm_subscriber_connection *conn);
+void assignment_fsm_update_id(struct gsm_subscriber_connection *conn);
