@@ -19,6 +19,10 @@
 #include <osmocom/core/fsm.h>
 #include <osmocom/core/tdef.h>
 #include <osmocom/core/time_cc.h>
+#include <osmocom/core/linuxlist.h>
+#include <osmocom/core/hashtable.h>
+#include <osmocom/core/linuxrbtree.h>
+#include <osmocom/core/utils.h>
 
 #include <osmocom/crypt/auth.h>
 
@@ -31,13 +35,29 @@
 #include <osmocom/gsm/protocol/gsm_12_21.h>
 #include <osmocom/gsm/protocol/gsm_03_41.h>
 #include <osmocom/abis/e1_input.h>
+#include <osmocom/bsc/bts_setup_ramp.h>
 #include <osmocom/bsc/meas_rep.h>
 #include <osmocom/bsc/acc.h>
 #include <osmocom/bsc/osmux.h>
 #include <osmocom/bsc/chan_counts.h>
+#include <osmocom/bsc/lchan.h>
 
 #define GSM_T3122_DEFAULT 10
 
+#define GSM_T3105_DEFAULT 100UL
+#define GSM_T3124_SDCCH 675UL
+#define GSM_T3124_OTHER_CH 320UL
+#define GSM_T3124_MAX GSM_T3124_SDCCH
+
+/* Some guess for delta (see comment below) */
+#define GSM_NY1_REQ_DELTA 1000UL
+/* Requirements: We want Ny1 to be as low as possible, while respecting T3105 * Ny1 > T3124 + delta
+ * with delta = time between expiration of T3124 and receiving HANDOVER FAILURE by the serving BSC. */
+#define GSM_NY1_DEFAULT ((unsigned long)((GSM_T3124_MAX + GSM_NY1_REQ_DELTA)/GSM_T3105_DEFAULT + 1))
+
+#define SCCP_CONN_ID_UNSET	0xFFFFFFFF
+#define SCCP_CONN_ID_MAX	0x00FFFFFE
+
 struct mgcp_client_conf;
 struct mgcp_client;
 struct gsm0808_cell_id;
@@ -78,6 +98,13 @@ struct smlc_config;
 #define DLCI2RSL_LINK_ID(dlci) \
 	((dlci & 0xc0) == 0xc0 ? 0x40 : 0x00) | (dlci & 0x07)
 
+typedef uint16_t gsm_bts_nr_t; /* See (struct gsm_bts *)->nr */
+/* BTS_NR_MAX = ((2 << ((sizeof(gsm_bts_nr_t) * 8) - 1)) - 1)
+ * This is needed as a constant so that the value can be stringified properly: */
+#define BTS_NR_MAX 65535
+#define BTS_NR_MAX_STR OSMO_STRINGIFY_VAL(BTS_NR_MAX)
+#define BTS_NR_VTY_ARG_VAL "<0-" BTS_NR_MAX_STR ">"
+
 /* 3-bit long values */
 #define EARFCN_PRIO_INVALID 8
 #define EARFCN_MEAS_BW_INVALID 8
@@ -91,22 +118,6 @@ typedef int gsm_cbfn(unsigned int hooknum,
 		     struct msgb *msg,
 		     void *data, void *param);
 
-/* Maximum number of neighbor cells whose average we track */
-#define MAX_NEIGH_MEAS		10
-/* Maximum size of the averaging window for neighbor cells */
-#define MAX_WIN_NEIGH_AVG	10
-/* Maximum number of report history we store */
-#define MAX_MEAS_REP		10
-
-/* processed neighbor measurements for one cell */
-struct neigh_meas_proc {
-	uint16_t arfcn;
-	uint8_t bsic;
-	uint8_t rxlev[MAX_WIN_NEIGH_AVG];
-	unsigned int rxlev_cnt;
-	uint8_t last_seen_nr;
-};
-
 struct gsm_classmark {
 	bool classmark1_set;
 	struct gsm48_classmark1 classmark1;
@@ -122,34 +133,6 @@ enum subscr_sccp_state {
 	SUBSCR_SCCP_ST_CONNECTED
 };
 
-enum channel_rate {
-	CH_RATE_SDCCH,
-	CH_RATE_HALF,
-	CH_RATE_FULL,
-};
-
-enum channel_rate chan_t_to_chan_rate(enum gsm_chan_t chan_t);
-
-enum lchan_csd_mode {
-	LCHAN_CSD_M_NT,
-	LCHAN_CSD_M_T_1200_75,
-	LCHAN_CSD_M_T_600,
-	LCHAN_CSD_M_T_1200,
-	LCHAN_CSD_M_T_2400,
-	LCHAN_CSD_M_T_9600,
-	LCHAN_CSD_M_T_14400,
-	LCHAN_CSD_M_T_29000,
-	LCHAN_CSD_M_T_32000,
-};
-
-struct channel_mode_and_rate {
-	enum gsm48_chan_mode chan_mode;
-	enum channel_rate chan_rate;
-	uint16_t s15_s0;
-	/* only used for GSM48_CMODE_DATA_* */
-	enum lchan_csd_mode csd_mode;
-};
-
 enum assign_for {
 	ASSIGN_FOR_NONE,
 	ASSIGN_FOR_BSSMAP_REQ,
@@ -161,14 +144,6 @@ extern const struct value_string assign_for_names[];
 static inline const char *assign_for_name(enum assign_for assign_for)
 { return get_value_string(assign_for_names, assign_for); }
 
-/* If .present is false, use the default value defined elsewhere. If true, use .val below.
- * (A practical benefit of this is that the default initialization sets .present to false, so that even if a .val == 0
- * is a valid value, a struct containing this as member does not need to explicitly set .val = INVALID_VAL_CONSTANT.) */
-struct optional_val {
-	bool present;
-	int val;
-};
-
 /* Information retrieved during an Assignment Request from the MSC. This is storage of the Assignment instructions
  * parsed from the Assignment Request message, to pass on until the gscon and assignment FSMs have decided whether an
  * Assignment is actually going to be carried out. Should remain unchanged after initial decoding. */
@@ -176,6 +151,7 @@ struct assignment_request {
 	enum assign_for assign_for;
 
 	bool aoip;
+	bool vgcs;
 
 	uint16_t msc_assigned_cic;
 
@@ -183,6 +159,7 @@ struct assignment_request {
 	uint16_t msc_rtp_port;
 	bool use_osmux;
 	uint8_t osmux_cid;
+	uint8_t rtp_extensions;
 
 	/* Rate/codec setting in preference order (need at least 1 !) */
 	int n_ch_mode_rate;
@@ -202,6 +179,9 @@ struct assignment_request {
 	/* The TSC to use if 'use' is true, otherwise automatically determine the TSC value to use. Valid range is 0 to
 	 * 7, as described in 3GPP TS 45.002. */
 	struct optional_val tsc;
+
+	/* The "speech / data indicator" from 3GPP TS 48.008 § 3.2.2.11 Channel Type (speech/data/signalling). */
+	enum gsm0808_chan_indicator ch_indctr;
 };
 
 /* State of an ongoing Assignment, while the assignment_fsm is still busy. This serves as state separation to keep the
@@ -215,7 +195,7 @@ struct assignment_fsm_data {
 	 */
 	struct assignment_request req;
 
-	bool requires_voice_stream;
+	enum gsm0808_chan_indicator ch_indctr;
 	struct channel_mode_and_rate selected_ch_mode_rate;
 
 	struct osmo_fsm_inst *fi;
@@ -312,6 +292,21 @@ struct handover {
 	struct osmo_mgcpc_ep_ci *created_ci_for_msc;
 };
 
+struct gsm_subscriber_connection;
+
+struct bscp_sccp_conn_node {
+	/* entry in (struct bsc_sccp_inst)->connections */
+	struct rb_node node;
+	/* Sigtran connection ID:
+	*  if set: Range (0..SCCP_CONN_ID_MAX) (24 bit)
+	*  if unset: SCCP_CONN_ID_UNSET (-1) if unset */
+	uint32_t conn_id;
+	/* backpointer where this sccp conn belongs to: */
+	struct gsm_subscriber_connection *gscon;
+};
+
+void bscp_sccp_conn_node_init(struct bscp_sccp_conn_node *sccp_conn, struct gsm_subscriber_connection *gscon);
+
 /* active radio connection of a mobile subscriber */
 struct gsm_subscriber_connection {
 	/* global linked list of subscriber_connections */
@@ -359,14 +354,12 @@ struct gsm_subscriber_connection {
 	/* flag to prevent multiple simultaneous ciphering commands */
 	int ciphering_handled;
 
-	/* SCCP connection associatd with this subscriber_connection */
+	/* SCCP connection associated with this subscriber_connection */
 	struct {
 		/* SCCP connection related */
 		struct bsc_msc_data *msc;
-
-		/* Sigtran connection ID */
-		int conn_id;
 		enum subscr_sccp_state state;
+		struct bscp_sccp_conn_node conn;
 	} sccp;
 
 	/* for audio handling */
@@ -376,6 +369,7 @@ struct gsm_subscriber_connection {
 		/* RTP address where the MSC expects us to send the RTP stream coming from the BTS. */
 		char msc_assigned_rtp_addr[INET6_ADDRSTRLEN];
 		uint16_t msc_assigned_rtp_port;
+		uint8_t rtp_extensions;
 
 		/* The endpoint at the MGW used to join both BTS and MSC side connections, e.g.
 		 * "rtpbridge/23@mgw". */
@@ -415,8 +409,8 @@ struct gsm_subscriber_connection {
 
 		/* Lb interface to the SMLC: BSSMAP-LE/SCCP connection associated with this subscriber */
 		struct {
-			int conn_id;
 			enum subscr_sccp_state state;
+			struct bscp_sccp_conn_node conn;
 		} lb;
 	} lcs;
 
@@ -430,6 +424,60 @@ struct gsm_subscriber_connection {
 	} fast_return;
 
 	enum gsm0808_cause clear_cause;
+
+	/* VGCS/VBS "call controling" connection */
+	struct {
+		/* Features supported by MSC/BSC */
+		bool ff_present;
+		struct gsm0808_vgcs_feature_flags ff;
+		/* Group Call Reference IE */
+		struct gsm0808_group_callref gc_ie;
+		enum gsm0808_service_flag sf;
+		uint32_t call_ref;
+		/* Call (BSC) FSM */
+		struct osmo_fsm_inst *fi;
+		/* Current talker */
+		struct gsm_subscriber_connection *talker;
+		/* L3 info of link establihment (Talker established) */
+		struct llist_head l3_queue;
+		/* Flag and cause (Talker released) */
+		bool talker_rel;
+		uint8_t talker_cause;
+		/* Flag that states acknowledgement of the talker by MSC */
+		bool msc_ack;
+		/* List of VGCS/VBS "resource controling" connections */
+		struct llist_head chan_list;
+	} vgcs_call;
+
+	/* VGCS/VBS "resource controling" connection */
+	struct {
+		/* List entry of chan_list of "call controling" connection */
+		struct llist_head list;
+		/* Group Call Reference IE */
+		struct gsm0808_group_callref gc_ie;
+		enum gsm0808_service_flag sf;
+		uint32_t call_ref;
+		/* Channel type IE */
+		struct gsm0808_channel_type ct;
+		/* Channel mode and rate */
+		struct channel_mode_and_rate ch_mode_rate;
+		/* Cell Identifier IE */
+		struct gsm0808_cell_id ci;
+		char ci_str[16];
+		/* Assignment Requirements IE */
+		enum gsm0808_assignment_requirement ar;
+		/* Call Identifier IE */
+		uint32_t call_id;
+		/* Pointer to VGCS/VBS "call controling" gsconn */
+		struct gsm_subscriber_connection *call;
+		/* Cell (BTS) FSM */
+		struct osmo_fsm_inst *fi;
+		/* lchan to be assigned */
+		struct gsm_lchan *new_lchan;
+		/* MGW peer */
+		char msc_rtp_addr[INET6_ADDRSTRLEN];
+		uint16_t msc_rtp_port;
+	} vgcs_chan;
 };
 
 
@@ -444,21 +492,6 @@ struct osmo_bsc_data;
 
 struct osmo_bsc_sccp_con;
 
-/* Channel Request reason */
-enum gsm_chreq_reason_t {
-	GSM_CHREQ_REASON_EMERG,
-	GSM_CHREQ_REASON_PAG,
-	GSM_CHREQ_REASON_CALL,
-	GSM_CHREQ_REASON_LOCATION_UPD,
-	GSM_CHREQ_REASON_OTHER,
-	GSM_CHREQ_REASON_PDCH,
-};
-
-static inline bool gsm_chreq_reason_is_voicecall(enum gsm_chreq_reason_t reason)
-{
-	return reason == GSM_CHREQ_REASON_EMERG || reason == GSM_CHREQ_REASON_CALL;
-}
-
 /* lchans 0..3 are SDCCH in combined channel configuration,
    use 4 as magic number for BCCH hack - see osmo-bts-../oml.c:opstart_compl() */
 #define CCCH_LCHAN 4
@@ -504,16 +537,24 @@ struct gsm_abis_mo {
 	struct abis_om_obj_inst obj_inst;
 	const char *name;
 	struct gsm_nm_state nm_state;
-	struct tlv_parsed *nm_attr;
 	struct gsm_bts *bts;
 	struct osmo_fsm_inst *fi;
+	bool sw_act_rep_received;
 	bool opstart_sent;
 	bool adm_unlock_sent;
 	bool get_attr_sent;
 	bool get_attr_rep_received;
 	bool set_attr_sent;
 	bool set_attr_ack_received;
+	bool rsl_connect_sent;
+	bool rsl_connect_ack_received;
 	bool force_rf_lock;
+	/* vendor specific fields below */
+	union {
+		struct {
+			uint8_t obj_version;
+		} ipaccess;
+	};
 };
 
 /* Ericsson OM2000 Managed Object */
@@ -547,20 +588,15 @@ struct om2k_mo {
  *
  * These macros convert from n to the other representations:
  */
-#define ALG_A5_NR_TO_RSL(A5_N) ((A5_N) >= 0 ? (A5_N)+1 : 0)
+#define ALG_A5_NR_TO_RSL(A5_N) ((int)(A5_N) >= 0 ? (A5_N)+1 : 0)
 #define ALG_A5_NR_TO_BSSAP(A5_N) ALG_A5_NR_TO_RSL(A5_N)
-#define ALG_A5_NR_TO_PERM_ALG_BITS(A5_N) ((A5_N) >= 0 ? 1<<(A5_N) : 0)
+#define ALG_A5_NR_TO_PERM_ALG_BITS(A5_N) ((int)(A5_N) >= 0 ? 1<<(A5_N) : 0)
 
 /* Up to 16 SI2quater are multiplexed; each fits 3 EARFCNS, so the practical maximum is 3*16.
  * The real maximum that fits in a total of 16 SI2quater rest octets also depends on the bits left by other SI2quater
  * rest octets elements, so to really fit 48 EARFCNs most other SI2quater elements need to be omitted. */
 #define MAX_EARFCN_LIST (3*16)
 
-/* is the data link established? who established it? */
-#define LCHAN_SAPI_UNUSED	0
-#define LCHAN_SAPI_MS		1
-#define LCHAN_SAPI_NET		2
-
 /* BTS ONLY */
 #define MAX_NUM_UL_MEAS	104
 #define LC_UL_M_F_L1_VALID	(1 << 0)
@@ -599,33 +635,6 @@ struct amr_multirate_conf {
 };
 /* /BTS ONLY */
 
-/* State of the SAPIs in the lchan */
-enum lchan_sapi_state {
-	LCHAN_SAPI_S_NONE,
-	LCHAN_SAPI_S_REQ,
-	LCHAN_SAPI_S_ASSIGNED,
-	LCHAN_SAPI_S_REL,
-	LCHAN_SAPI_S_ERROR,
-};
-
-#define MAX_A5_KEY_LEN	(128/8)
-
-struct gsm_encr {
-	uint8_t alg_a5_n; /* N: 0 (A5/0), 1 (A5/1), ... 7 (A5/7) */
-	uint8_t key_len;
-	uint8_t key[MAX_A5_KEY_LEN];
-	bool kc128_present;
-	uint8_t kc128[16];
-};
-
-#define LOGPLCHAN(lchan, ss, level, fmt, args...) \
-	LOGP(ss, level, "%s (ss=%d,%s) (%s) " fmt, \
-	     lchan ? gsm_ts_and_pchan_name(lchan->ts) : "-", \
-	     lchan ? lchan->nr : 0, \
-	     lchan ? gsm_lchant_name(lchan->type) : "-", \
-	     bsc_subscr_name(lchan && lchan->conn ? lchan->conn->bsub : NULL), \
-	     ## args)
-
 /* Iterate at most N lchans of the given timeslot.
  * usage:
  * struct gsm_lchan *lchan;
@@ -641,260 +650,9 @@ struct gsm_encr {
 	     && ((lchan - (ts)->lchan) < (N)); \
 	     lchan++)
 
-enum lchan_activate_for {
-	ACTIVATE_FOR_NONE,
-	ACTIVATE_FOR_MS_CHANNEL_REQUEST,
-	ACTIVATE_FOR_ASSIGNMENT,
-	ACTIVATE_FOR_HANDOVER,
-	ACTIVATE_FOR_VTY,
-	ACTIVATE_FOR_MODE_MODIFY_RTP,
-};
-
-extern const struct value_string lchan_activate_mode_names[];
-static inline const char *lchan_activate_mode_name(enum lchan_activate_for activ_for)
-{ return get_value_string(lchan_activate_mode_names, activ_for); }
-
-enum imm_ass_time {
-	IMM_ASS_TIME_POST_CHAN_ACK = 0,
-	IMM_ASS_TIME_PRE_CHAN_ACK,
-	IMM_ASS_TIME_PRE_TS_ACK,
-};
-
-struct lchan_activate_info {
-	enum lchan_activate_for activ_for;
-	/* If activ_for == ACTIVATE_FOR_MS_CHANNEL_REQUEST, the original CHREQ reason. */
-	enum gsm_chreq_reason_t chreq_reason;
-	struct gsm_subscriber_connection *for_conn;
-	struct channel_mode_and_rate ch_mode_rate;
-	struct gsm_encr encr;
-	bool requires_voice_stream;
-	bool wait_before_switching_rtp; /*< true = requires LCHAN_EV_READY_TO_SWITCH_RTP */
-	uint16_t msc_assigned_cic;
-	/* During intra-BSC handover, we keep the MGW endpoint intact and just re-route to the new lchan. This
-	 * activate_info is for the new lchan, the re_use_mgw_endpoint_from_lchan points at the old lchan. */
-	struct gsm_lchan *re_use_mgw_endpoint_from_lchan;
-	bool ta_known;
-	uint8_t ta;
-
-	/* The TSC Set to use if 'use' is true, otherwise automatically determine the TSC Set value to use. Valid range
-	 * is 1 to 4, as described in 3GPP TS 45.002. */
-	struct optional_val tsc_set;
-	/* The TSC to use if 'use' is true, otherwise automatically determine the TSC value to use. Valid range is 0 to
-	 * 7, as described in 3GPP TS 45.002. */
-	struct optional_val tsc;
-
-	bool vamos;
-
-	/* A copy of bts->imm_ass_time at the time where Channel Activation was requested. A change in the VTY
-	 * configuration has immediate effect on the value, so make sure we don't get mixed up when it gets changed
-	 * while a channel activation is in progress. */
-	enum imm_ass_time imm_ass_time;
-};
-
-enum lchan_modify_for {
-	MODIFY_FOR_NONE,
-	MODIFY_FOR_ASSIGNMENT,
-	MODIFY_FOR_VTY,
-};
-
-extern const struct value_string lchan_modify_for_names[];
-static inline const char *lchan_modify_for_name(enum lchan_modify_for modify_for)
-{ return get_value_string(lchan_modify_for_names, modify_for); }
-
-struct lchan_modify_info {
-	enum lchan_modify_for modify_for;
-	struct channel_mode_and_rate ch_mode_rate;
-	bool requires_voice_stream;
-	uint16_t msc_assigned_cic;
-
-	/* The TSC Set to use if 'use' is true, otherwise automatically determine the TSC Set value to use. Valid range
-	 * is 1 to 4, as described in 3GPP TS 45.002. */
-	struct optional_val tsc_set;
-	/* The TSC to use if 'use' is true, otherwise automatically determine the TSC value to use. Valid range is 0 to
-	 * 7, as described in 3GPP TS 45.002. */
-	struct optional_val tsc;
-
-	bool vamos;
-};
-
 #define INTERF_DBM_UNKNOWN 0
 #define INTERF_BAND_UNKNOWN 0xff
 
-/* Measurement pre-processing state */
-struct gsm_power_ctrl_meas_proc_state {
-	/* Number of measurements processed */
-	unsigned int meas_num;
-	/* Algorithm specific data */
-	union {
-		struct {
-			/* Scaled up 100 times average value */
-			int Avg100;
-		} ewma;
-	};
-};
-
-struct lchan_power_ctrl_state {
-	/* Measurement pre-processing state (for dynamic mode) */
-	struct gsm_power_ctrl_meas_proc_state rxlev_meas_proc;
-	struct gsm_power_ctrl_meas_proc_state rxqual_meas_proc;
-	/* Number of SACCH blocks to skip (for dynamic mode) */
-	int skip_block_num;
-};
-
-struct gsm_lchan {
-	/* The TS that we're part of */
-	struct gsm_bts_trx_ts *ts;
-	/* The logical subslot number in the TS */
-	uint8_t nr;
-	char *name;
-
-	char *last_error;
-
-	struct osmo_fsm_inst *fi;
-	struct osmo_fsm_inst *fi_rtp;
-	struct osmo_mgcpc_ep_ci *mgw_endpoint_ci_bts;
-
-	struct {
-		/* The request as made by the caller, see lchan_activate().
-		 * lchan->activate.info is treated immutable: remains unchanged throughout the Activation.
-		 * The mutable versions are below: some values need automatic adjustments, in which case they are copied
-		 * from immutable lchan->activate.info.* to lchan->activate.*, for example lchan->activate.ch_mode_rate
-		 * is initially a copy of lchan->activate.info.ch_mode_rate, and is possibly adjusted afterwards. */
-		struct lchan_activate_info info;
-
-		struct channel_mode_and_rate ch_mode_rate;
-		struct gsm48_multi_rate_conf mr_conf_filtered;
-		bool activ_ack; /*< true as soon as RSL Chan Activ Ack is received */
-		bool immediate_assignment_sent;
-		/*! This flag ensures that when an lchan activation has succeeded, and we have already
-		 * sent ACKs like Immediate Assignment or BSSMAP Assignment Complete, and if other errors
-		 * occur later, e.g. during release, that we don't send a NACK out of context. */
-		bool concluded;
-		enum gsm0808_cause gsm0808_error_cause;
-		/* Actually used TSC Set. */
-		int tsc_set;
-		/* Actually used TSC. */
-		uint8_t tsc;
-	} activate;
-
-	struct {
-		/* The request as made by the caller, see lchan_mode_modify().
-		 * lchan->modify.info is treated immutable: remains unchanged throughout the Mode Modify.
-		 * The mutable versions are below: some values need automatic adjustments, in which case they are copied
-		 * from immutable lchan->modify.info.* to lchan->modify.*, for example lchan->modify.ch_mode_rate
-		 * is initially a copy of lchan->modify.info.ch_mode_rate, and is possibly adjusted afterwards. */
-		struct lchan_modify_info info;
-
-		struct channel_mode_and_rate ch_mode_rate;
-		struct gsm48_multi_rate_conf mr_conf_filtered;
-		/* Actually used TSC Set. */
-		int tsc_set;
-		/* Actually used TSC. */
-		uint8_t tsc;
-		bool concluded;
-	} modify;
-
-	struct {
-		/* If an event to release the lchan comes in while still waiting for responses, just mark this
-		 * flag, so that the lchan will gracefully release at the next sensible junction. */
-		bool requested;
-		bool do_rr_release;
-		enum gsm48_rr_cause rr_cause;
-		bool last_eutran_plmn_valid;
-		struct osmo_plmn_id last_eutran_plmn;
-
-		/* There is an RSL error cause of value 0, so we need a separate flag. */
-		bool in_error;
-		/* RSL error code, RSL_ERR_* */
-		uint8_t rsl_error_cause;
-
-		/* If a release event is being handled, ignore other ricocheting release events until that
-		 * release handling has concluded. */
-		bool in_release_handler;
-	} release;
-
-	/* The logical channel type */
-	enum gsm_chan_t type;
-	/* Power levels for MS and BTS */
-	uint8_t bs_power_db;
-	uint8_t ms_power;
-	/* Encryption information */
-	struct gsm_encr encr;
-
-	/* Established data link layer services */
-	uint8_t sapis[8];
-
-	struct {
-		uint32_t bound_ip; /*< where the BTS receives RTP */
-		uint16_t bound_port;
-		uint32_t connect_ip; /*< where the BTS sends RTP to (MGW) */
-		uint16_t connect_port;
-		uint16_t conn_id;
-		uint8_t rtp_payload;
-		uint8_t rtp_payload2;
-		uint8_t speech_mode;
-
-		/* info we need to postpone the AoIP
-		 * assignment completed message */
-		struct {
-			uint8_t rr_cause;
-			bool valid;
-		} ass_compl;
-	} abis_ip;
-
-	/* At first, the Timing Advance from the initial Channel Request. Later, the Timing Advance value received from
-	 * the most recent Measurement Report. */
-	uint8_t last_ta;
-
-	/* table of neighbor cell measurements */
-	struct neigh_meas_proc neigh_meas[MAX_NEIGH_MEAS];
-
-	/* cache of last measurement reports on this lchan */
-	struct gsm_meas_rep meas_rep[MAX_MEAS_REP];
-	int meas_rep_idx;
-	int meas_rep_count;
-	uint8_t meas_rep_last_seen_nr;
-
-	/* GSM Random Access data */
-	/* TODO: don't allocate this, rather keep an "is_present" flag */
-	struct gsm48_req_ref *rqd_ref;
-
-	struct gsm_subscriber_connection *conn;
-
-	/* After the Channel Activation ACK or RSL Mode Modify ACK is received, this reflects the actually used
-	 * channel_mode_and_rate. */
-	struct channel_mode_and_rate current_ch_mode_rate;
-	struct gsm48_multi_rate_conf current_mr_conf;
-
-	/* Circuit-Switched TSC Set in use, or -1 if no specific TSC Set was requested. The valid range is 1-4 as
-	 * described in the spec 3GPP TS 45.002. */
-	int tsc_set;
-	/* Training Sequence Code in use. The valid range is 0-7 as described in the spec 3GPP TS 45.002. */
-	uint8_t tsc;
-
-	struct {
-		/* Whether this lchan represents a secondary "shadow" lchan to multiplex a second MS onto a primary
-		 * "normal" lchan */
-		bool is_secondary;
-
-		/* Whether this lchan is activated/modified into a mode that allows VAMOS multiplexing at this moment */
-		bool enabled;
-	} vamos;
-
-	/* dBm value of interference level as reported in the most recent Resource Indication, if any for this lchan. Or
-	 * INTERF_DBM_UNKNOWN if this lchan was not included in the most recent Resource Indication.
-	 * The range is typically -115 to -85 dBm, here stored 1:1 as a signed integer, to ease comparison. */
-	int16_t interf_dbm;
-	/* Actual reported interference band index, or INTERF_BAND_UNKNOWN if this lchan was not included in the most
-	 * recent Resource Indication. */
-	uint8_t interf_band;
-	/* MS power control state */
-	struct lchan_power_ctrl_state ms_power_ctrl;
-	/* Timestamps and markers to track active state duration. */
-	struct timespec active_start;
-	struct timespec active_stored;
-};
-
 /* One Timeslot in a TRX */
 struct gsm_bts_trx_ts {
 	struct gsm_bts_trx *trx;
@@ -927,7 +685,6 @@ struct gsm_bts_trx_ts {
 	bool is_rsl_ready;
 
 	struct gsm_abis_mo mo;
-	struct tlv_parsed nm_attr;
 	uint8_t nm_chan_comb;
 	int tsc;		/* -1 == use BTS TSC */
 
@@ -967,8 +724,6 @@ struct gsm_bts_trx_ts {
 	struct chan_counts chan_counts;
 };
 
-#define GSM_LCHAN_SI(lchan, i) (void *)((lchan)->si.buf[i][0])
-
 struct gsm_envabtse {
 	struct gsm_abis_mo mo;
 };
@@ -1135,7 +890,7 @@ extern struct osmo_tdef_group bsc_tdef_group[];
 
 struct gsm_network *gsm_network_init(void *ctx);
 
-struct gsm_bts *gsm_bts_num(const struct gsm_network *net, int num);
+struct gsm_bts *gsm_bts_num(const struct gsm_network *net, gsm_bts_nr_t num);
 struct gsm_bts *gsm_bts_by_cell_id(const struct gsm_network *net,
 				   const struct gsm0808_cell_id *cell_id,
 				   int match_idx);
@@ -1143,28 +898,18 @@ struct gsm_bts *gsm_bts_by_cell_id(const struct gsm_network *net,
 extern const struct value_string gsm_chreq_descs[];
 extern const struct value_string gsm_pchant_names[];
 extern const struct value_string gsm_pchant_descs[];
-extern const struct value_string gsm_pchan_ids[];
 const char *gsm_pchan_name(enum gsm_phys_chan_config c);
-static inline const char *gsm_pchan_id(enum gsm_phys_chan_config c)
-{ return get_value_string(gsm_pchan_ids, c); }
 enum gsm_phys_chan_config gsm_pchan_parse(const char *name);
-const char *gsm_lchant_name(enum gsm_chan_t c);
 const char *gsm_chreq_name(enum gsm_chreq_reason_t c);
 char *gsm_ts_name(const struct gsm_bts_trx_ts *ts);
 char *gsm_ts_and_pchan_name(const struct gsm_bts_trx_ts *ts);
-void lchan_update_name(struct gsm_lchan *lchan);
-uint64_t gsm_lchan_active_duration_ms(const struct gsm_lchan *lchan);
-
-static inline char *gsm_lchan_name(const struct gsm_lchan *lchan)
-{
-	OSMO_ASSERT(lchan);
-	return lchan->name;
-}
 
 void gsm_abis_mo_reset(struct gsm_abis_mo *mo);
 void gsm_mo_init(struct gsm_abis_mo *mo, struct gsm_bts *bts,
 		 uint8_t obj_class, uint8_t p1, uint8_t p2, uint8_t p3);
 
+struct gsm_abis_mo *gsm_objclass2mo(struct gsm_bts *bts, uint8_t obj_class,
+				    const struct abis_om_obj_inst *obj_inst);
 struct gsm_nm_state *
 gsm_objclass2nmstate(struct gsm_bts *bts, uint8_t obj_class,
 		 const struct abis_om_obj_inst *obj_inst);
@@ -1191,13 +936,9 @@ uint8_t pchan_subslots(enum gsm_phys_chan_config pchan);
 uint8_t pchan_subslots_vamos(enum gsm_phys_chan_config pchan);
 bool ts_is_tch(struct gsm_bts_trx_ts *ts);
 
-struct gsm_lchan *gsm_lchan_vamos_to_primary(const struct gsm_lchan *lchan_vamos);
-struct gsm_lchan *gsm_lchan_primary_to_vamos(const struct gsm_lchan *lchan_primary);
-
 struct gsm_bts *conn_get_bts(struct gsm_subscriber_connection *conn);
 
 void conn_update_ms_power_class(struct gsm_subscriber_connection *conn, uint8_t power_class);
-void lchan_update_ms_power_ctrl_level(struct gsm_lchan *lchan, int ms_power_dbm);
 
 struct gsm_tz {
 	int override; /* if 0, use system's time zone instead. */
@@ -1213,6 +954,19 @@ struct all_allocated {
 	struct osmo_time_cc static_tch;
 };
 
+struct bsc_sccp_inst {
+	struct osmo_sccp_instance *sccp; /* backpointer */
+	/* rbtree root of 'struct bscp_sccp_conn_node' in this instance, ordered by conn_id */
+	struct rb_root connections;
+	uint32_t next_id; /* next id to allocate */
+};
+
+struct bsc_sccp_inst *bsc_sccp_inst_alloc(void *ctx);
+uint32_t bsc_sccp_inst_next_conn_id(struct bsc_sccp_inst *bsc_sccp);
+int bsc_sccp_inst_register_gscon(struct bsc_sccp_inst *bsc_sccp, struct bscp_sccp_conn_node *sccp_conn);
+void bsc_sccp_inst_unregister_gscon(struct bsc_sccp_inst *bsc_sccp, struct bscp_sccp_conn_node *sccp_conn);
+struct gsm_subscriber_connection *bsc_sccp_inst_get_gscon_by_conn_id(const struct bsc_sccp_inst *bsc_sccp, uint32_t conn_id);
+
 struct gsm_network {
 	struct osmo_plmn_id plmn;
 
@@ -1230,14 +984,19 @@ struct gsm_network {
 	struct rate_ctr_group *bsc_ctrs;
 	struct osmo_stat_item_group *bsc_statg;
 
-	unsigned int num_bts;
+	gsm_bts_nr_t num_bts;
 	struct llist_head bts_list;
 	struct llist_head bts_rejected;
+	DECLARE_HASHTABLE(bts_by_nr, 10);
+	DECLARE_HASHTABLE(bts_by_lac, 10);
+	DECLARE_HASHTABLE(bts_by_lac_ci, 10);
+#define LAC_CI_HASHTABLE_KEY(lac, ci) ((((uint32_t)(ci)) << sizeof(lac)) | (uint32_t)(lac))
+	DECLARE_HASHTABLE(bts_by_ci, 10);
 
-        /* BTS-based counters when we can't find the actual BTS
-         * e.g. when conn->lchan is NULL */
-        struct rate_ctr_group *bts_unknown_ctrs;
-        struct osmo_stat_item_group *bts_unknown_statg;
+	/* BTS-based counters when we can't find the actual BTS
+	 * e.g. when conn->lchan is NULL */
+	struct rate_ctr_group *bts_unknown_ctrs;
+	struct osmo_stat_item_group *bts_unknown_statg;
 
 	/* see gsm_network_T_defs */
 	struct osmo_tdef *T_defs;
@@ -1249,10 +1008,10 @@ struct gsm_network {
 
 	/* msc configuration */
 	struct llist_head mscs;
-	uint8_t mscs_round_robin_next_nr;
+	unsigned int mscs_round_robin_next_nr;
 	/* Emergency calls potentially select a different set of MSCs, so to not mess up the normal round-robin
 	 * behavior, emergency calls need a separate round-robin counter. */
-	uint8_t mscs_round_robin_next_emerg_nr;
+	unsigned int mscs_round_robin_next_emerg_nr;
 
 	/* rf ctl related bits */
 	int mid_call_timeout;
@@ -1278,13 +1037,8 @@ struct gsm_network {
 	 * OsmoMSC, this should be tied to the location area code (LAC). */
 	struct gsm_tz tz;
 
-	/* List of all struct bsc_subscr used in libbsc. This llist_head is
-	 * allocated so that the llist_head pointer itself can serve as a
-	 * talloc context (useful to not have to pass the entire gsm_network
-	 * struct to the bsc_subscr_* API, and for bsc_susbscr unit tests to
-	 * not require gsm_data.h). In an MSC-without-BSC environment, this
-	 * pointer is NULL to indicate absence of a bsc_subscribers list. */
-	struct llist_head *bsc_subscribers;
+	/* Keeps track of struct bsc_subcr. */
+	struct bsc_subscr_store *bsc_subscribers;
 
 	/* Timer for periodic channel load measurements to maintain each BTS's T3122. */
 	struct osmo_timer_list t3122_chan_load_timer;
@@ -1318,6 +1072,8 @@ struct gsm_network {
 	/* Don't refuse to start with mutually exclusive codec settings */
 	bool allow_unusable_timeslots;
 
+	struct bts_setup_ramp_net bts_setup_ramp;
+
 	uint8_t nri_bitlen;
 	struct osmo_nri_ranges *null_nri_ranges;
 
@@ -1325,6 +1081,11 @@ struct gsm_network {
 
 	struct chan_counts chan_counts;
 	struct all_allocated all_allocated;
+
+	/* PCU socket state */
+	char *pcu_sock_path;
+	unsigned int pcu_sock_wqueue_len_max;
+	struct pcu_sock_state *pcu_state;
 };
 
 struct gsm_audio_support {
@@ -1332,12 +1093,12 @@ struct gsm_audio_support {
                 ver : 7;
 };
 
+int gsm_audio_support_cmp(const struct gsm_audio_support *a, const struct gsm_audio_support *b);
+
 extern void talloc_ctx_init(void *ctx_root);
 
 enum gsm_bts_type parse_btstype(const char *arg);
 const char *btstype2str(enum gsm_bts_type type);
-struct gsm_bts *gsm_bts_by_lac(struct gsm_network *net, unsigned int lac,
-				struct gsm_bts *start_bts);
 
 extern void *tall_bsc_ctx;
 
@@ -1360,9 +1121,6 @@ void set_ts_e1link(struct gsm_bts_trx_ts *ts, uint8_t e1_nr,
 /* generic E1 line operations for all ISDN-based BTS. */
 extern struct e1inp_line_ops bts_isdn_e1inp_line_ops;
 
-/* control interface handling */
-int bsc_base_ctrl_cmds_install(void);
-
 bool ts_is_usable(const struct gsm_bts_trx_ts *ts);
 
 int gsm_lchan_type_by_pchan(enum gsm_phys_chan_config pchan);
@@ -1371,8 +1129,6 @@ enum gsm_phys_chan_config gsm_pchan_by_lchan_type(enum gsm_chan_t type);
 enum gsm48_rr_cause bsc_gsm48_rr_cause_from_gsm0808_cause(enum gsm0808_cause c);
 enum gsm48_rr_cause bsc_gsm48_rr_cause_from_rsl_cause(uint8_t c);
 
-int bsc_sccp_inst_next_conn_id(struct osmo_sccp_instance *sccp);
-
 /* Interference Measurement Parameters */
 struct gsm_interf_meas_params {
 	/* Intave: Interference Averaging period (see 3GPP TS 45.008, table A.1) */
