@@ -63,11 +63,6 @@ struct oml_node_state {
 	struct con_group *cg;
 };
 
-static int dummy_config_write(struct vty *v)
-{
-	return CMD_SUCCESS;
-}
-
 /* FIXME: auto-generate those strings from the value_string lists */
 #define OM2K_OBJCLASS_VTY "(trxc|tg|ts|tf|is|con|dp|mctr|cf|tx|rx)"
 #define OM2K_OBJCLASS_VTY_HELP 	"TRX Controller\n"	\
@@ -81,9 +76,10 @@ static int dummy_config_write(struct vty *v)
 				"Central Function\n"	\
 				"Transmitter\n"		\
 				"Receiver\n"
+#define OM2K_VTY_HELP "Configure OM2K specific parameters\n"
 
 DEFUN(om2k_class_inst, om2k_class_inst_cmd,
-	"bts <0-255> om2000 class " OM2K_OBJCLASS_VTY
+	"bts " BTS_NR_VTY_ARG_VAL " om2000 class " OM2K_OBJCLASS_VTY
 					" <0-255> <0-255> <0-255>",
 	"BTS related commands\n" "BTS Number\n"
 	"Manipulate the OM2000 managed objects\n"
@@ -124,7 +120,7 @@ DEFUN(om2k_class_inst, om2k_class_inst_cmd,
 }
 
 DEFUN(om2k_classnum_inst, om2k_classnum_inst_cmd,
-	"bts <0-255> om2000 class <0-255> <0-255> <0-255> <0-255>",
+	"bts " BTS_NR_VTY_ARG_VAL " om2000 class <0-255> <0-255> <0-255> <0-255>",
 	"BTS related commands\n" "BTS Number\n"
 	"Manipulate the OML managed objects\n"
 	"Object Class\n" "Object Class\n"
@@ -475,10 +471,31 @@ DEFUN_USRATTR(cfg_bts_alt_mode, cfg_bts_alt_mode_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN_USRATTR(cfg_bts_om2k_sync, cfg_bts_om2k_sync_cmd,
+	      X(BSC_VTY_ATTR_RESTART_ABIS_OML_LINK),
+	      "om2000 sync-source (internal|external)",
+	      OM2K_VTY_HELP
+	      "TF Synchronization Source\n"
+	      "Use Internal (E1)\n"
+	      "USe External (GPS)\n")
+{
+	struct gsm_bts *bts = vty->index;
+	if (bts->type != GSM_BTS_TYPE_RBS2000) {
+		vty_out(vty, "%% Command only works for RBS2000%s",
+			VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+	if (!strcmp(argv[0], "internal"))
+		bts->rbs2000.sync_src = OM2K_SYNC_SRC_INTERNAL;
+	else if (!strcmp(argv[0], "external"))
+		bts->rbs2000.sync_src = OM2K_SYNC_SRC_EXTERNAL;
+	return CMD_SUCCESS;
+}
+
 DEFUN_USRATTR(cfg_bts_om2k_version_limit, cfg_bts_om2k_version_limit_cmd,
 	      X(BSC_VTY_ATTR_RESTART_ABIS_OML_LINK),
 	      "om2000 version-limit (oml|rsl) gen <0-99> rev <0-99>",
-	      "Configure OM2K specific parameters\n"
+	      OM2K_VTY_HELP
 	      "Configure optional maximum protocol version to negotiate\n"
 	      "Limit OML IWD version\n" "Limit RSL IWD version\n"
 	      "Generation limit\n"
@@ -552,6 +569,33 @@ DEFUN_USRATTR(cfg_bts_is_conn_list, cfg_bts_is_conn_list_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN_USRATTR(cfg_trx_om2k_rx_diversity,
+		  cfg_trx_om2k_rx_diversity_cmd,
+		  X(BSC_VTY_ATTR_RESTART_ABIS_RSL_LINK),
+		  "om2000 rx-diversity-mode (a|ab|b)",
+		  OM2K_VTY_HELP
+		  "RX Diversity\n"
+		  "Antenna TX/RX (A)\n"
+		  "Both Antennas\n"
+		  "Antenna RX (B)\n")
+
+{
+	struct gsm_bts_trx *trx = vty->index;
+
+	if (trx->bts->type != GSM_BTS_TYPE_RBS2000) {
+		vty_out(vty, "%% Command only works for RBS2000%s",
+			VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if (!strcmp(argv[0], "a"))
+		trx->rbs2000.rx_diversity = OM2K_RX_DIVERSITY_A;
+	else if (!strcmp(argv[0], "ab"))
+		trx->rbs2000.rx_diversity = OM2K_RX_DIVERSITY_AB;
+	else if (!strcmp(argv[0], "b"))
+		trx->rbs2000.rx_diversity = OM2K_RX_DIVERSITY_B;
+	return CMD_SUCCESS;
+}
 
 DEFUN(om2k_conf_req, om2k_conf_req_cmd,
 	"configuration-request",
@@ -631,6 +675,24 @@ static void dump_con_group(struct vty *vty, struct con_group *cg)
 	}
 }
 
+static const struct value_string om2k_rx_diversity_names[4] = {
+	{ OM2K_RX_DIVERSITY_A,		"a" },
+	{ OM2K_RX_DIVERSITY_AB,		"ab" },
+	{ OM2K_RX_DIVERSITY_B,		"b" },
+	{ 0, NULL }
+};
+
+static const char *rx_diversity2str(enum om2k_rx_diversity type)
+{
+	return get_value_string(om2k_rx_diversity_names, type);
+}
+
+void abis_om2k_config_write_trx(struct vty *vty, struct gsm_bts_trx *trx)
+{
+	vty_out(vty, "   om2000 rx-diversity-mode %s%s",
+		rx_diversity2str(trx->rbs2000.rx_diversity), VTY_NEWLINE);
+}
+
 void abis_om2k_config_write_bts(struct vty *vty, struct gsm_bts *bts)
 {
 	struct is_conn_group *igrp;
@@ -656,6 +718,10 @@ void abis_om2k_config_write_bts(struct vty *vty, struct gsm_bts *bts)
 				(bts->rbs2000.om2k_version[i].limit >> 8),
 				(bts->rbs2000.om2k_version[i].limit & 0xff),
 				VTY_NEWLINE);
+	vty_out(vty, "  om2000 sync-source %s%s",
+		bts->rbs2000.sync_src != OM2K_SYNC_SRC_EXTERNAL
+		? "internal" : "external",
+		VTY_NEWLINE);
 }
 
 static void vty_dump_om2k_mo(struct vty *vty, const struct om2k_mo *mo, const char *pfx)
@@ -682,7 +748,7 @@ static void vty_dump_om2k_mo(struct vty *vty, const struct om2k_mo *mo, const ch
 }
 
 DEFUN(show_om2k_mo, show_om2k_mo_cmd,
-	"show bts <0-255> om2k-mo",
+	"show bts " BTS_NR_VTY_ARG_VAL " om2k-mo",
 	SHOW_STR "Display information about a BTS\n"
 	"BTS number\n" "OM2000 Managed Object information\n")
 {
@@ -755,9 +821,12 @@ int abis_om2k_vty_init(void)
 
 	install_element(BTS_NODE, &cfg_bts_is_conn_list_cmd);
 	install_element(BTS_NODE, &cfg_bts_alt_mode_cmd);
+	install_element(BTS_NODE, &cfg_bts_om2k_sync_cmd);
 	install_element(BTS_NODE, &cfg_bts_om2k_version_limit_cmd);
 	install_element(BTS_NODE, &cfg_om2k_con_group_cmd);
 	install_element(BTS_NODE, &del_om2k_con_group_cmd);
 
+	install_element(TRX_NODE, &cfg_trx_om2k_rx_diversity_cmd);
+
 	return 0;
 }
