@@ -49,7 +49,7 @@ const struct value_string osmo_bts_variant_names[_NUM_BTS_VARIANT + 1] = {
 	{ BTS_OSMO_LITECELL15,	"osmo-bts-lc15" },
 	{ BTS_OSMO_OCTPHY,	"osmo-bts-octphy" },
 	{ BTS_OSMO_SYSMO,	"osmo-bts-sysmo" },
-	{ BTS_OSMO_TRX,		"omso-bts-trx" },
+	{ BTS_OSMO_TRX,		"osmo-bts-trx" },
 	{ 0, NULL }
 };
 
@@ -117,8 +117,20 @@ int gsm_bts_model_register(struct gsm_bts_model *model)
 	return 0;
 }
 
-static const uint8_t bts_cell_timer_default[] =
-				{ 3, 3, 3, 3, 3, 10, 3, 10, 3, 10, 3 };
+static const uint8_t bts_cell_timer_default[11] = {
+	3,	/* blocking timer (T1) */
+	3,	/* blocking retries */
+	3,	/* unblocking retries */
+	3,	/* reset timer (T2) */
+	3,	/* reset retries */
+	10,	/* suspend timer (T3) in 100ms */
+	3,	/* suspend retries */
+	10,	/* resume timer (T4) in 100ms */
+	3,	/* resume retries */
+	10,	/* capability update timer (T5) */
+	 3,	/* capability update retries */
+};
+
 static const struct gprs_rlc_cfg rlc_cfg_default = {
 	.parameter = {
 		[RLC_T3142] = 20,
@@ -144,7 +156,19 @@ static const struct gprs_rlc_cfg rlc_cfg_default = {
 
 static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
 {
+	/* Entries in bts->loc_list are freed by talloc recursively, no need to free them here. */
+
+	llist_del(&bts->list);
+	hash_del(&bts->node_by_nr);
+	hash_del(&bts->node_by_lac);
+	hash_del(&bts->node_by_lac_ci);
+	hash_del(&bts->node_by_ci);
+
 	paging_destructor(bts);
+	bts_setup_ramp_remove(bts);
+
+	acc_ramp_deinit(&bts->acc_ramp);
+	acc_mgr_deinit(&bts->acc_mgr);
 
 	osmo_timer_del(&bts->cbch_timer);
 
@@ -169,7 +193,7 @@ static int gsm_bts_talloc_destructor(struct gsm_bts *bts)
  * This part is shared among the thin programs in osmo-bsc/src/utils/.
  * osmo-bsc requires further initialization that pulls in more dependencies (see
  * bsc_bts_alloc_register()). */
-struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm, uint8_t bts_num)
+struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm, gsm_bts_nr_t bts_num)
 {
 	struct gsm_bts *bts = talloc_zero(bts_sm, struct gsm_bts);
 
@@ -179,6 +203,17 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	talloc_set_destructor(bts, gsm_bts_talloc_destructor);
 
 	bts->nr = bts_num;
+
+	llist_add_tail(&bts->list, &net->bts_list);
+	hash_add(net->bts_by_nr, &bts->node_by_nr, bts->nr);
+	net->num_bts++;
+
+	/* Default bts->location_area_code == GSM_LAC_RESERVED_DETACHED, don't add to hashtable: */
+	INIT_HLIST_NODE(&bts->node_by_lac);
+	INIT_HLIST_NODE(&bts->node_by_lac_ci);
+	/* Default CI = 0: */
+	hash_add(net->bts_by_ci, &bts->node_by_ci, bts->cell_identity);
+
 	bts->num_trx = 0;
 	INIT_LLIST_HEAD(&bts->trx_list);
 	bts->network = net;
@@ -190,7 +225,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	bts->mo.fi = osmo_fsm_inst_alloc(&nm_bts_fsm, bts, bts,
 					      LOGL_INFO, NULL);
 	osmo_fsm_inst_update_id_f(bts->mo.fi, "bts%d", bts->nr);
-	gsm_mo_init(&bts->mo, bts, NM_OC_BTS, bts->nr, 0xff, 0xff);
+	gsm_mo_init(&bts->mo, bts, NM_OC_BTS, bts->bts_nr, 0xff, 0xff);
 
 	/* 3GPP TS 08.18, chapter 5.4.1: 0 is reserved for signalling */
 	bts->gprs.cell.bvci = 2;
@@ -202,7 +237,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 						   &bts->gprs.cell, LOGL_INFO, NULL);
 	osmo_fsm_inst_update_id_f(bts->gprs.cell.mo.fi, "gprs-cell%d", bts->nr);
 	gsm_mo_init(&bts->gprs.cell.mo, bts, NM_OC_GPRS_CELL,
-			bts->nr, 0xff, 0xff);
+		    bts->bts_nr, 0xff, 0xff);
 
 	/* init statistics */
 	bts->bts_ctrs = rate_ctr_group_alloc(bts, &bts_ctrg_desc, bts->nr);
@@ -292,6 +327,9 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	bts->early_classmark_allowed_3g = true; /* 3g Early Classmark Sending controlled by bts->early_classmark_allowed param */
 	bts->si_unused_send_empty = true;
 	bts->chan_alloc_tch_signalling_policy = BTS_TCH_SIGNALLING_ALWAYS;
+	bts->chan_alloc_dyn_params.ul_rxlev_thresh = 50; /* >= -60 dBm */
+	bts->chan_alloc_dyn_params.ul_rxlev_avg_num = 2; /* at least 2 samples */
+	bts->chan_alloc_dyn_params.c0_chan_load_thresh = 60; /* >= 60% */
 	bts->si_common.cell_sel_par.cell_resel_hyst = 2; /* 4 dB */
 	bts->si_common.cell_sel_par.rxlev_acc_min = 0;
 	bts->si_common.si2quater_neigh_list.arfcn = bts->si_common.data.earfcn_list;
@@ -327,6 +365,10 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	INIT_LLIST_HEAD(&bts->neighbors);
 	INIT_LLIST_HEAD(&bts->oml_fail_rep);
 	INIT_LLIST_HEAD(&bts->chan_rqd_queue);
+	INIT_LLIST_HEAD(&bts->depends_on);
+
+	/* Don't pin the BTS to any MGW by default: */
+	bts->mgw_pool_target = -1;
 
 	/* Enable all codecs by default. These get reset to a more fine grained selection IF a
 	 * 'codec-support' config appears in the config file (see bsc_vty.c). */
@@ -425,9 +467,12 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	memcpy(&bts->mr_half.bts_mode[0], &amr_hr_ms_bts_mode[0], sizeof(amr_hr_ms_bts_mode));
 	bts->mr_half.num_modes = ARRAY_SIZE(amr_hr_ms_bts_mode);
 
+	bts->use_osmux = OSMUX_USAGE_OFF;
+
 	bts_cbch_init(bts);
 	bts_etws_init(bts);
 
+	bts_setup_ramp_init_bts(bts);
 	acc_mgr_init(&bts->acc_mgr, bts);
 	acc_ramp_init(&bts->acc_ramp, bts);
 
@@ -448,6 +493,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 	bts->interf_meas_params_cfg = interf_meas_params_def;
 
 	bts->rach_max_delay = 63;
+	bts->rach_expiry_timeout = 32;
 
 	/* SRVCC is enabled by default */
 	bts->srvcc_fast_return_allowed = true;
@@ -456,7 +502,7 @@ struct gsm_bts *gsm_bts_alloc(struct gsm_network *net, struct gsm_bts_sm *bts_sm
 }
 
 /* Validate BTS configuration (ARFCN settings and physical channel configuration) */
-int gsm_bts_check_cfg(struct gsm_bts *bts)
+__attribute__((weak)) int gsm_bts_check_cfg(struct gsm_bts *bts)
 {
 	struct gsm_bts_trx *trx;
 
@@ -504,6 +550,13 @@ int gsm_bts_check_cfg(struct gsm_bts *bts)
 			     bts_gprs_mode_name(bts->gprs.mode));
 			return -EINVAL;
 		}
+		if (bts->use_osmux == OSMUX_USAGE_ONLY &&
+		    !osmo_bts_has_feature(&bts->features, BTS_FEAT_OSMUX)) {
+			LOGP(DNM, LOGL_ERROR,
+			     "(bts=%u) osmux use set to 'only', but BTS does not support Osmux\n",
+			     bts->nr);
+			return -EINVAL;
+		}
 	}
 
 	/* Verify the physical channel mapping */
@@ -515,6 +568,9 @@ int gsm_bts_check_cfg(struct gsm_bts *bts)
 		}
 	}
 
+	if (!gsm_bts_check_ny1(bts))
+		return -EINVAL;
+
 	return 0;
 }
 
@@ -733,7 +789,8 @@ struct gsm_bts_trx *gsm_bts_trx_num(const struct gsm_bts *bts, int num)
 
 void bts_store_uptime(struct gsm_bts *bts)
 {
-	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_UPTIME_SECONDS), bts_uptime(bts));
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_UPTIME_SECONDS),
+			   bts->oml_link ? bts_updowntime(bts) : 0);
 }
 
 void bts_store_lchan_durations(struct gsm_bts *bts)
@@ -791,20 +848,20 @@ void bts_store_lchan_durations(struct gsm_bts *bts)
 	rate_ctr_add(rate_ctr_group_get_ctr(bts->bts_ctrs, BTS_CTR_CHAN_SDCCH_ACTIVE_MILLISECONDS_TOTAL), elapsed_sdcch_ms);
 }
 
-unsigned long long bts_uptime(const struct gsm_bts *bts)
+unsigned long long bts_updowntime(const struct gsm_bts *bts)
 {
 	struct timespec tp;
 
-	if (!bts->uptime || !bts->oml_link)
+	if (!bts->updowntime)
 		return 0;
 
 	if (osmo_clock_gettime(CLOCK_MONOTONIC, &tp) != 0) {
-		LOGP(DNM, LOGL_ERROR, "BTS %u uptime computation failure: %s\n", bts->nr, strerror(errno));
+		LOGP(DNM, LOGL_ERROR, "BTS %u uptime/downtime computation failure: %s\n", bts->nr, strerror(errno));
 		return 0;
 	}
 
 	/* monotonic clock helps to ensure that the conversion is valid */
-	return difftime(tp.tv_sec, bts->uptime);
+	return difftime(tp.tv_sec, bts->updowntime);
 }
 
 char *get_model_oml_status(const struct gsm_bts *bts)
@@ -835,52 +892,58 @@ void gsm_bts_mo_reset(struct gsm_bts *bts)
 	}
 }
 
-/* Assume there are only 256 possible bts */
-osmo_static_assert(sizeof(((struct gsm_bts *) 0)->nr) == 1, _bts_nr_is_256);
-static void depends_calc_index_bit(int bts_nr, int *idx, int *bit)
+int bts_depend_mark(struct gsm_bts *bts, gsm_bts_nr_t dep)
 {
-	*idx = bts_nr / (8 * 4);
-	*bit = bts_nr % (8 * 4);
+	struct bts_depends_on_entry *entry;
+	entry = talloc_zero(bts, struct bts_depends_on_entry);
+	if (!entry) {
+		LOG_BTS(bts, DNM, LOGL_ERROR, "Alloc of struct bts_depends_on_entry failed");
+		return -1;
+	}
+	entry->bts_nr = dep;
+	llist_add_tail(&entry->list, &bts->depends_on);
+	return 0;
 }
 
-void bts_depend_mark(struct gsm_bts *bts, int dep)
+static struct bts_depends_on_entry *bts_depend_find_entry(const struct gsm_bts *bts, gsm_bts_nr_t dep)
 {
-	int idx, bit;
-	depends_calc_index_bit(dep, &idx, &bit);
-
-	bts->depends_on[idx] |= 1U << bit;
+	struct bts_depends_on_entry *entry;
+	llist_for_each_entry(entry, &bts->trx_list, list) {
+		if (entry->bts_nr == dep)
+			return entry;
+	}
+	return NULL;
 }
 
-void bts_depend_clear(struct gsm_bts *bts, int dep)
+void bts_depend_clear(struct gsm_bts *bts, gsm_bts_nr_t dep)
 {
-	int idx, bit;
-	depends_calc_index_bit(dep, &idx, &bit);
-
-	bts->depends_on[idx] &= ~(1U << bit);
+	struct bts_depends_on_entry *entry;
+	entry = bts_depend_find_entry(bts, dep);
+	if (!entry)
+		return;
+	llist_del(&entry->list);
+	talloc_free(entry);
 }
 
-int bts_depend_is_depedency(struct gsm_bts *base, struct gsm_bts *other)
+bool bts_depend_is_depedency(struct gsm_bts *base, struct gsm_bts *other)
 {
-	int idx, bit;
-	depends_calc_index_bit(other->nr, &idx, &bit);
-
-	/* Check if there is a depends bit */
-	return (base->depends_on[idx] & (1U << bit)) > 0;
+	struct bts_depends_on_entry *entry = bts_depend_find_entry(base, other->nr);
+	return !!entry;
 }
 
-static int bts_is_online(struct gsm_bts *bts)
+static bool bts_is_online(const struct gsm_bts *bts)
 {
 	/* TODO: support E1 BTS too */
-	if (!is_ipaccess_bts(bts))
-		return 1;
+	if (!is_ipa_abisip_bts(bts))
+		return true;
 
 	if (!bts->oml_link)
-		return 0;
+		return false;
 
 	return bts->mo.nm_state.operational == NM_OPSTATE_ENABLED;
 }
 
-int bts_depend_check(struct gsm_bts *bts)
+bool bts_depend_check(struct gsm_bts *bts)
 {
 	struct gsm_bts *other_bts;
 
@@ -889,9 +952,9 @@ int bts_depend_check(struct gsm_bts *bts)
 			continue;
 		if (bts_is_online(other_bts))
 			continue;
-		return 0;
+		return false;
 	}
-	return 1;
+	return true;
 }
 
 /* get the radio link timeout (based on SACCH decode errors, according
@@ -958,21 +1021,32 @@ int gsm_bts_set_system_infos(struct gsm_bts *bts)
 	return 0;
 }
 
+/* Send the given C0 power reduction value to the BTS */
+int gsm_bts_send_c0_power_red(const struct gsm_bts *bts, const uint8_t red)
+{
+	if (!bts_is_online(bts))
+		return -ENOTCONN;
+	if (!osmo_bts_has_feature(&bts->features, BTS_FEAT_BCCH_POWER_RED))
+		return -ENOTSUP;
+	if (bts->model->power_ctrl_send_c0_power_red == NULL)
+		return -ENOTSUP;
+	return bts->model->power_ctrl_send_c0_power_red(bts, red);
+}
+
+/* Send the given C0 power reduction value to the BTS and update the internal state */
 int gsm_bts_set_c0_power_red(struct gsm_bts *bts, const uint8_t red)
 {
 	struct gsm_bts_trx *c0 = bts->c0;
 	unsigned int tn;
 	int rc;
 
-	if (!osmo_bts_has_feature(&bts->features, BTS_FEAT_BCCH_POWER_RED))
-		return -ENOTSUP;
-	if (bts->model->power_ctrl_set_c0_power_red == NULL)
-		return -ENOTSUP;
-
-	rc = bts->model->power_ctrl_set_c0_power_red(bts, red);
+	rc = gsm_bts_send_c0_power_red(bts, red);
 	if (rc != 0)
 		return rc;
 
+	LOG_BTS(bts, DRSL, LOGL_NOTICE, "%sabling BCCH carrier power reduction "
+		"operation mode (maximum %u dB)\n", red ? "En" : "Dis", red);
+
 	/* Timeslot 0 is always transmitting BCCH/CCCH */
 	c0->ts[0].c0_max_power_red_db = 0;
 
@@ -983,7 +1057,7 @@ int gsm_bts_set_c0_power_red(struct gsm_bts *bts, const uint8_t red)
 		switch (ts->pchan_is) {
 		/* Not allowed on CCCH/BCCH */
 		case GSM_PCHAN_CCCH:
-			/* Preceeding timeslot shall not exceed 2 dB */
+			/* Preceding timeslot shall not exceed 2 dB */
 			if (prev->c0_max_power_red_db > 0)
 				prev->c0_max_power_red_db = 2;
 			/* fall-through */
@@ -1025,7 +1099,9 @@ void gsm_bts_stats_reset(struct gsm_bts *bts)
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_CHAN_SDCCH8_CBCH_TOTAL), 0);
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_CHAN_OSMO_DYN_USED), 0);
 	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_CHAN_OSMO_DYN_TOTAL), 0);
-	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_T3113), 0);
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_PAGING_T3113), 0);
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_PAGING_REQ_QUEUE_LENGTH), paging_pending_requests_nr(bts));
+	osmo_stat_item_set(osmo_stat_item_group_get_item(bts->bts_statg, BTS_STAT_PAGING_AVAILABLE_SLOTS), bts->paging.available_slots);
 }
 
 const struct rate_ctr_desc bts_ctr_description[] = {
@@ -1610,12 +1686,12 @@ const struct osmo_stat_item_desc bts_stat_desc[] = {
 		  "Number of SDCCH8 channels total",
 		  "", 60, 0 },
 	[BTS_STAT_CHAN_TCH_F_PDCH_USED] = \
-		{ "chan_tch_f_pdch:used",
-		  "Number of TCH/F_PDCH channels used",
+		{ "chan_dynamic_ipaccess:used",
+		  "Number of DYNAMIC/IPACCESS channels used",
 		  "", 60, 0 },
 	[BTS_STAT_CHAN_TCH_F_PDCH_TOTAL] = \
-		{ "chan_tch_f_pdch:total",
-		  "Number of TCH/F_PDCH channels total",
+		{ "chan_dynamic_ipaccess:total",
+		  "Number of DYNAMIC/IPACCESS channels total",
 		  "", 60, 0 },
 	[BTS_STAT_CHAN_CCCH_SDCCH4_CBCH_USED] = \
 		{ "chan_ccch_sdcch4_cbch:used",
@@ -1634,12 +1710,12 @@ const struct osmo_stat_item_desc bts_stat_desc[] = {
 		  "Number of SDCCH8+CBCH channels total",
 		  "", 60, 0 },
 	[BTS_STAT_CHAN_OSMO_DYN_USED] = \
-		{ "chan_osmo_dyn:used",
-		  "Number of TCH/F_TCH/H_SDCCH8_PDCH channels used",
+		{ "chan_dynamic_osmocom:used",
+		  "Number of DYNAMIC/OSMOCOM channels used",
 		  "", 60, 0 },
 	[BTS_STAT_CHAN_OSMO_DYN_TOTAL] = \
-		{ "chan_osmo_dyn:total",
-		  "Number of TCH/F_TCH/H_SDCCH8_PDCH channels total",
+		{ "chan_dynamic_osmocom:total",
+		  "Number of DYNAMIC/OSMOCOM channels total",
 		  "", 60, 0 },
 	[BTS_STAT_T3122] = \
 		{ "T3122",
@@ -1677,8 +1753,16 @@ const struct osmo_stat_item_desc bts_stat_desc[] = {
 		{ "num_trx:total",
 		  "Number of configured TRX in this BTS",
 		  "" },
-	[BTS_STAT_T3113] = \
-		{ "t3113",
+	[BTS_STAT_PAGING_REQ_QUEUE_LENGTH] = \
+		{ "paging:request_queue_length",
+		  "Paging Request queue length",
+		  "", 60, 0 },
+	[BTS_STAT_PAGING_AVAILABLE_SLOTS] = \
+		{ "paging:available_slots",
+		  "Available paging slots in this BTS",
+		  "", 60, 0 },
+	[BTS_STAT_PAGING_T3113] = \
+		{ "paging:t3113",
 		  "T3113 paging timer",
 		  "s", 60, 0 },
 };
@@ -1690,3 +1774,20 @@ const struct osmo_stat_item_group_desc bts_statg_desc = {
 	.num_items = ARRAY_SIZE(bts_stat_desc),
 	.item_desc = bts_stat_desc,
 };
+
+/* Return 'true' if and only if Ny1 satisfies network requirements */
+bool gsm_bts_check_ny1(const struct gsm_bts *bts)
+{
+	unsigned long T3105, ny1, ny1_recommended;
+	T3105 = osmo_tdef_get(bts->network->T_defs, 3105, OSMO_TDEF_MS, -1);
+	ny1 = osmo_tdef_get(bts->network->T_defs, -3105, OSMO_TDEF_CUSTOM, -1);
+	if (!(T3105 * ny1 > GSM_T3124_MAX + GSM_NY1_REQ_DELTA)) {
+		/* See comment for GSM_NY1_DEFAULT */
+		ny1_recommended = (GSM_T3124_MAX + GSM_NY1_REQ_DELTA)/T3105 + 1;
+		LOGP(DNM, LOGL_ERROR, "Value of Ny1 should be higher. "
+				     "Is: %lu, lowest recommendation: %lu\n",
+		     ny1, ny1_recommended);
+		return false;
+	}
+	return true;
+}
