@@ -46,7 +46,7 @@
 #include <osmocom/bsc/bsc_msc_data.h>
 #include <osmocom/bsc/system_information.h>
 #include <osmocom/bsc/bts.h>
-
+#include <osmocom/bsc/lchan.h>
 
 int gsm48_sendmsg(struct msgb *msg)
 {
@@ -57,6 +57,15 @@ int gsm48_sendmsg(struct msgb *msg)
 	return rsl_data_request(msg, 0);
 }
 
+int gsm48_sendmsg_unit(struct msgb *msg)
+{
+	if (msg->lchan)
+		msg->dst = rsl_chan_link(msg->lchan);
+
+	msg->l3h = msg->data;
+	return rsl_unit_data_request(msg, 0);
+}
+
 /* Section 9.1.8 / Table 9.9 */
 struct chreq {
 	uint8_t val;
@@ -299,12 +308,15 @@ static int generate_cell_sel_ind_after_rel(uint8_t *out, unsigned int out_len, c
 	}
 }
 
+#define REPEAT_RR_RELEASE_UI	3
+
 /* 7.1.7 and 9.1.7: RR CHANnel RELease */
-int gsm48_send_rr_release(struct gsm_lchan *lchan)
+int gsm48_send_rr_release(struct gsm_lchan *lchan, bool ui)
 {
-	struct msgb *msg = gsm48_msgb_alloc_name("GSM 04.08 RR REL");
+	struct msgb *msg = gsm48_msgb_alloc_name("GSM 04.08 RR REL"), *msgc;
 	struct gsm48_hdr *gh = (struct gsm48_hdr *) msgb_put(msg, sizeof(*gh));
 	uint8_t *cause;
+	int n;
 
 	msg->lchan = lchan;
 	gh->proto_discr = GSM48_PDISC_RR;
@@ -329,8 +341,17 @@ int gsm48_send_rr_release(struct gsm_lchan *lchan)
 	       gsm_lchan_name(lchan),  lchan->release.rr_cause,
 	       rr_cause_name(lchan->release.rr_cause));
 
-	/* Send actual release request to MS */
-	return gsm48_sendmsg(msg);
+	/* Send actual release request to MS (dedicated channel) */
+	if (!ui)
+		return gsm48_sendmsg(msg);
+
+	/* Send actual release request to MS (VGCS channel) */
+	for (n = 1; n < REPEAT_RR_RELEASE_UI; n++) {
+		msgc = msgb_copy(msg, "CHAN RELEASE copy");
+		msgc->lchan = lchan;
+		gsm48_sendmsg_unit(msgc);
+	}
+	return gsm48_sendmsg_unit(msg);
 }
 
 int send_siemens_mrpci(struct gsm_lchan *lchan,
@@ -565,7 +586,7 @@ struct msgb *gsm48_make_ho_cmd(const struct gsm_lchan *new_lchan,
 		 * TODO: NCI (Normal cell indication), currently 0. */
 		const uint8_t sync_ind = async ? 0x00 : 0x01;
 		/* T (4 bit) + V (4 bit), see 3GPP TS 44.018, 10.5.2.39 */
-		msgb_v_put(msg, GSM48_IE_SYNC_IND | (sync_ind & 0x0f));
+		msgb_v_put(msg, (GSM48_IE_SYNC_IND_HO << 4) | (sync_ind & 0x0f));
 	}
 
 	if (new_lchan->ts->hopping.enabled) {
@@ -595,7 +616,7 @@ struct msgb *gsm48_make_ho_cmd(const struct gsm_lchan *new_lchan,
 		if (new_lchan->encr.alg_a5_n > 0)
 			cms = (new_lchan->encr.alg_a5_n - 1) << 1 | 1;
 		/* T (4 bit) + V (4 bit), see 3GPP TS 44.018, 10.5.2.9 */
-		msgb_v_put(msg, GSM48_IE_CIP_MODE_SET | (cms & 0x0f));
+		msgb_v_put(msg, (GSM48_IE_CIP_MODE_SET_HO << 4) | (cms & 0x0f));
 	}
 
 	/* in case of multi rate we need to attach a config */
@@ -737,7 +758,7 @@ int gsm48_lchan_modify(struct gsm_lchan *lchan, uint8_t mode)
 		}
 	}
 
-	if (lchan->modify.info.vamos && lchan->modify.tsc_set > 0) {
+	if (lchan->modify.info.type_for == LCHAN_TYPE_FOR_VAMOS && lchan->modify.tsc_set > 0) {
 		/* Add the Extended TSC Set IE. So far we only need a TSC Set sent for VAMOS.
 		 * Convert from spec conforming "human readable" TSC Set 1-4 to 0-3 on the wire */
 		msgb_tv_put(msg, GSM48_IE_EXTENDED_TSC_SET, (lchan->modify.tsc_set - 1) & 0x3);
@@ -746,6 +767,66 @@ int gsm48_lchan_modify(struct gsm_lchan *lchan, uint8_t mode)
 	return gsm48_sendmsg(msg);
 }
 
+/* TS 44.018 section 9.1.48 */
+int gsm48_send_uplink_release(struct gsm_lchan *lchan, uint8_t cause)
+{
+	struct msgb *msg = gsm48_msgb_alloc_name("GSM 04.08 UL RELEASE");
+	struct gsm48_hdr *gh = (struct gsm48_hdr *) msgb_put(msg, sizeof(*gh));
+
+	msg->lchan = lchan;
+	gh->proto_discr = GSM48_PDISC_RR;
+	gh->msg_type = GSM48_MT_RR_UPLINK_RELEASE;
+
+	msgb_put_u8(msg, cause);
+
+	return gsm48_sendmsg(msg);
+}
+
+/* TS 44.018 section 9.1.46 */
+int gsm48_send_uplink_busy(struct gsm_lchan *lchan)
+{
+	struct msgb *msg = gsm48_msgb_alloc_name("GSM 04.08 UL BUSY");
+	struct gsm48_hdr *gh = (struct gsm48_hdr *) msgb_put(msg, sizeof(*gh));
+
+	msg->lchan = lchan;
+	gh->proto_discr = GSM48_PDISC_RR;
+	gh->msg_type = GSM48_MT_RR_UPLINK_BUSY;
+
+	return gsm48_sendmsg_unit(msg);
+}
+
+/* TS 44.018 section 9.1.47 */
+int gsm48_send_uplink_free(struct gsm_lchan *lchan, uint8_t acc_bit, uint8_t *uic)
+{
+	struct msgb *msg = gsm48_msgb_alloc_name("GSM 04.08 UL FREE");
+	struct gsm48_hdr_sh *sh = (struct gsm48_hdr_sh *) msgb_put(msg, sizeof(*sh) + 22);
+	struct bitvec *bv = bitvec_alloc(22, NULL);
+
+	msg->lchan = lchan;
+	sh->rr_short_pd = GSM48_PDISC_SH_RR;
+	sh->msg_type = GSM48_MT_RR_SH_UL_FREE;
+	sh->l2_header = 0;
+
+	/* < Uplink Access Request bit > */
+	bitvec_set_bit(bv, (acc_bit) ? H : L);
+
+	/* { L | H <Uplink Identity Code bit(6) > } */
+	if (uic) {
+		bitvec_set_bit(bv, H);
+		sh->data[0] = 0x40 | *uic;
+	} else
+		bitvec_set_bit(bv, L);
+
+	/* Emergency mode not supported */
+
+	/* Padding the rest with 0x2B and apply to msg. */
+	bitvec_spare_padding(bv, 175);
+	bitvec_pack(bv, sh->data);
+	bitvec_free(bv);
+
+	return gsm48_sendmsg_unit(msg);
+}
+
 int gsm48_rx_rr_modif_ack(struct msgb *msg)
 {
 	struct gsm48_hdr *gh = msgb_l3(msg);
@@ -766,13 +847,63 @@ int gsm48_rx_rr_modif_ack(struct msgb *msg)
 	return 0;
 }
 
+/* Get the ARFCN from the BCCH channel list by the index "BCCH-FREQ-NCELL i"
+ * (idx) as described in 3GPP TS 144.018 § 10.5.2.20. The BCCH channel list is
+ * split into two sub lists, each ascendingly ordered by ARFCN > 0:
+ *   1) SI* and SI*bis entries
+ *   2) SI*ter entries (if available)
+ * ARFCN 0 is at the end of each sub list.
+ * Both sub lists are stored in one bitvec (nbv), iterate twice through it. */
+int neigh_list_get_arfcn(struct gsm_bts *bts, const struct bitvec *nbv, unsigned int idx)
+{
+	unsigned int arfcn, i = 0;
+
+	/* First sub list, ARFCN > 0 */
+	for (arfcn = 1; arfcn < nbv->data_len * 8; arfcn++) {
+		if (bitvec_get_bit_pos(nbv, arfcn) == ZERO)
+			continue;
+		/* Skip SI*ter */
+		if (!band_compatible(bts, arfcn))
+			continue;
+		if (i == idx)
+			return arfcn;
+		i++;
+	}
+
+	/* First sub list, ARFCN == 0 (last position) */
+	if (bitvec_get_bit_pos(nbv, 0) == ONE && band_compatible(bts, 0)) {
+		if (i == idx)
+			return 0;
+		i++;
+	}
+
+	/* Second sub list, ARFCN > 0 */
+	for (arfcn = 1; arfcn < nbv->data_len * 8; arfcn++) {
+		if (bitvec_get_bit_pos(nbv, arfcn) == ZERO)
+			continue;
+		/* Require SI*ter */
+		if (band_compatible(bts, arfcn))
+			continue;
+		if (i == idx)
+			return arfcn;
+		i++;
+	}
+
+	/* Second sub list, ARFCN == 0 (last position) */
+	if (bitvec_get_bit_pos(nbv, 0) == ONE && !band_compatible(bts, 0) && i == idx)
+		return 0;
+
+	return -EINVAL;
+}
+
 int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 {
 	struct gsm48_hdr *gh = msgb_l3(msg);
 	uint8_t *data = gh->data;
 	struct gsm_bts *bts = msg->lchan->ts->trx->bts;
-	struct bitvec *nbv = &bts->si_common.neigh_list;
+	struct bitvec *nbv;
 	struct gsm_meas_rep_cell *mrc;
+	int rc;
 
 	if (gh->msg_type != GSM48_MT_RR_MEAS_REP)
 		return -EINVAL;
@@ -796,11 +927,21 @@ int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 		return 0;
 	}
 
+	/* If the phone reports BA-IND 1 this is a report for the SI5* set.
+	 * If we have generated SI5* with manual SI5 neighbor list, the measurements refer to it. */
+	if ((rep->flags & MEAS_REP_F_BA1) && bts->neigh_list_manual_mode == NL_MODE_MANUAL_SI5SEP)
+		nbv = &bts->si_common.si5_neigh_list;
+	else
+		nbv = &bts->si_common.neigh_list;
+
 	/* an encoding nightmare in perfection */
 	mrc = &rep->cell[0];
 	mrc->rxlev = data[3] & 0x3f;
 	mrc->neigh_idx = data[4] >> 3;
-	mrc->arfcn = bitvec_get_nth_set_bit(nbv, mrc->neigh_idx + 1);
+	rc = neigh_list_get_arfcn(bts, nbv, mrc->neigh_idx);
+	if (rc < 0)
+		goto error;
+	mrc->arfcn = rc;
 	mrc->bsic = ((data[4] & 0x07) << 3) | (data[5] >> 5);
 	if (rep->num_cell < 2)
 		return 0;
@@ -808,7 +949,10 @@ int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 	mrc = &rep->cell[1];
 	mrc->rxlev = ((data[5] & 0x1f) << 1) | (data[6] >> 7);
 	mrc->neigh_idx = (data[6] >> 2) & 0x1f;
-	mrc->arfcn = bitvec_get_nth_set_bit(nbv, mrc->neigh_idx + 1);
+	rc = neigh_list_get_arfcn(bts, nbv, mrc->neigh_idx);
+	if (rc < 0)
+		goto error;
+	mrc->arfcn = rc;
 	mrc->bsic = ((data[6] & 0x03) << 4) | (data[7] >> 4);
 	if (rep->num_cell < 3)
 		return 0;
@@ -816,7 +960,10 @@ int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 	mrc = &rep->cell[2];
 	mrc->rxlev = ((data[7] & 0x0f) << 2) | (data[8] >> 6);
 	mrc->neigh_idx = (data[8] >> 1) & 0x1f;
-	mrc->arfcn = bitvec_get_nth_set_bit(nbv, mrc->neigh_idx + 1);
+	rc = neigh_list_get_arfcn(bts, nbv, mrc->neigh_idx);
+	if (rc < 0)
+		goto error;
+	mrc->arfcn = rc;
 	mrc->bsic = ((data[8] & 0x01) << 5) | (data[9] >> 3);
 	if (rep->num_cell < 4)
 		return 0;
@@ -824,7 +971,10 @@ int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 	mrc = &rep->cell[3];
 	mrc->rxlev = ((data[9] & 0x07) << 3) | (data[10] >> 5);
 	mrc->neigh_idx = data[10] & 0x1f;
-	mrc->arfcn = bitvec_get_nth_set_bit(nbv, mrc->neigh_idx + 1);
+	rc = neigh_list_get_arfcn(bts, nbv, mrc->neigh_idx);
+	if (rc < 0)
+		goto error;
+	mrc->arfcn = rc;
 	mrc->bsic = data[11] >> 2;
 	if (rep->num_cell < 5)
 		return 0;
@@ -832,7 +982,10 @@ int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 	mrc = &rep->cell[4];
 	mrc->rxlev = ((data[11] & 0x03) << 4) | (data[12] >> 4);
 	mrc->neigh_idx = ((data[12] & 0xf) << 1) | (data[13] >> 7);
-	mrc->arfcn = bitvec_get_nth_set_bit(nbv, mrc->neigh_idx + 1);
+	rc = neigh_list_get_arfcn(bts, nbv, mrc->neigh_idx);
+	if (rc < 0)
+		goto error;
+	mrc->arfcn = rc;
 	mrc->bsic = (data[13] >> 1) & 0x3f;
 	if (rep->num_cell < 6)
 		return 0;
@@ -840,10 +993,18 @@ int gsm48_parse_meas_rep(struct gsm_meas_rep *rep, struct msgb *msg)
 	mrc = &rep->cell[5];
 	mrc->rxlev = ((data[13] & 0x01) << 5) | (data[14] >> 3);
 	mrc->neigh_idx = ((data[14] & 0x07) << 2) | (data[15] >> 6);
-	mrc->arfcn = bitvec_get_nth_set_bit(nbv, mrc->neigh_idx + 1);
+	rc = neigh_list_get_arfcn(bts, nbv, mrc->neigh_idx);
+	if (rc < 0)
+		goto error;
+	mrc->arfcn = rc;
 	mrc->bsic = data[15] & 0x3f;
 
 	return 0;
+
+error:
+	LOGP(DRR, LOGL_ERROR, "Invalid BCCH channel list index %d in measurement report\n", mrc->neigh_idx);
+	rep->num_cell = 0;
+	return 0;
 }
 
 /* 9.1.29 RR Status */
@@ -1051,6 +1212,12 @@ static void dispatch_dtap(struct gsm_subscriber_connection *conn,
 			 * according to spec */
 			bsc_dtap(conn, link_id, msg);
 			break;
+		case GSM48_MT_RR_UPLINK_RELEASE:
+			/* When the calling phone releases the uplink before it has been assigned to the group
+			 * channel, it will send an UPLINK RELEASE message on the dedicated channel. The MSC
+			 * has to take care of it. (assigning the phone to the group channel) */
+			bsc_dtap(conn, link_id, msg);
+			break;
 		default:
 			/* Drop unknown RR message */
 			LOG_LCHAN(msg->lchan, LOGL_NOTICE, "Unknown RR message: %s\n",
