@@ -377,7 +377,8 @@ int neighbor_address_resolution(const struct gsm_network *net, const struct cell
 	struct gsm_bts *local_neighbor = NULL;
 	struct gsm0808_cell_id_list2 remote_neighbors = { 0 };
 
-	llist_for_each_entry(bts_tmp, &net->bts_list, list) {
+	hash_for_each_possible(net->bts_by_lac_ci, bts_tmp, node_by_lac_ci,
+			       LAC_CI_HASHTABLE_KEY(lac, cell_id)) {
 		if (bts_tmp->location_area_code != lac)
 			continue;
 		if (bts_tmp->cell_identity != cell_id)
@@ -489,7 +490,7 @@ int neighbor_ctrl_cmds_install(struct gsm_network *net)
 
 struct ctrl_handle *neighbor_controlif_setup(struct gsm_network *net)
 {
-	/* See also osmobsc-usermanual.pdf, section 14.1 Neighbor Address Resolution Service */
+	/* DEPRECATED: see osmobsc-usermanual.pdf, section 16.1.1 Neighbor Address Resolution Service */
 	return ctrl_interface_setup_dynip2(net, net->neigh_ctrl.addr, net->neigh_ctrl.port,
 					   NULL, _LAST_CTRL_NODE_NEIGHBOR);
 }
