@@ -39,6 +39,7 @@
 #include <osmocom/bsc/handover_fsm.h>
 #include <osmocom/bsc/smscb.h>
 #include <osmocom/bsc/lb.h>
+#include <osmocom/bsc/meas_feed.h>
 
 #include <osmocom/ctrl/control_cmd.h>
 #include <osmocom/ctrl/control_if.h>
@@ -91,12 +92,12 @@ static const char *config_file = "osmo-bsc.cfg";
 static const char *rf_ctrl = NULL;
 static int daemonize = 0;
 
-static void print_usage()
+static void print_usage(void)
 {
 	printf("Usage: osmo-bsc\n");
 }
 
-static void print_help()
+static void print_help(void)
 {
 	printf("Some useful options:\n");
 	printf("  -h --help 			This text.\n");
@@ -231,7 +232,7 @@ static int oml_msg_nack(struct nm_nack_signal_data *nack)
 		return 0;
 	}
 
-	if (is_ipaccess_bts(nack->bts))
+	if (is_ipa_abisip_bts(nack->bts))
 		ipaccess_drop_oml_deferred(nack->bts);
 
 	return 0;
@@ -312,16 +313,16 @@ static void generate_ma_for_bts(struct gsm_bts *bts)
 
 static void bootstrap_rsl(struct gsm_bts_trx *trx)
 {
+	struct gsm_bts *bts = trx->bts;
 	unsigned int i;
 	int rc;
 
 	LOG_TRX(trx, DRSL, LOGL_NOTICE, "bootstrapping RSL "
 		"on ARFCN %u using MCC-MNC %s LAC=%u CID=%u BSIC=%u\n",
 		trx->arfcn, osmo_plmn_name(&bsc_gsmnet->plmn),
-		trx->bts->location_area_code,
-		trx->bts->cell_identity, trx->bts->bsic);
+		bts->location_area_code, bts->cell_identity, bts->bsic);
 
-	if (trx->bts->type == GSM_BTS_TYPE_NOKIA_SITE) {
+	if (bts->type == GSM_BTS_TYPE_NOKIA_SITE) {
 		rsl_nokia_si_begin(trx);
 	}
 
@@ -338,14 +339,14 @@ static void bootstrap_rsl(struct gsm_bts_trx *trx)
 		return;
 	}
 
-	if (trx->bts->type == GSM_BTS_TYPE_NOKIA_SITE) {
+	if (bts->type == GSM_BTS_TYPE_NOKIA_SITE) {
 		/* channel unspecific, power reduction in 2 dB steps */
 		rsl_bs_power_control(trx, 0xFF, trx->max_power_red / 2);
 		rsl_nokia_si_end(trx);
 	}
 
-	if (trx->bts->model->power_ctrl_send_def_params != NULL) {
-		rc = trx->bts->model->power_ctrl_send_def_params(trx);
+	if (bts->model->power_ctrl_send_def_params != NULL) {
+		rc = bts->model->power_ctrl_send_def_params(trx);
 		if (rc) {
 			LOG_TRX(trx, DRSL, LOGL_ERROR, "Failed to send default "
 				"MS/BS Power control parameters (rc=%d)\n", rc);
@@ -360,7 +361,7 @@ static void bootstrap_rsl(struct gsm_bts_trx *trx)
 	}
 
 	/* Drop all expired channel requests in the list */
-	abis_rsl_chan_rqd_queue_flush(trx->bts);
+	abis_rsl_chan_rqd_queue_flush(bts);
 }
 
 struct osmo_timer_list update_connection_stats_timer;
@@ -375,6 +376,28 @@ static void update_connection_stats_cb(void *data)
 	osmo_timer_schedule(&update_connection_stats_timer, 1, 0);
 }
 
+static bool nch_position_compatible_with_combined_ccch(const struct gsm_bts *bts)
+{
+	switch (bts->nch.num_blocks) {
+	case 0:
+		/* no NCH enabled, so we are fine */
+		return true;
+	case 1:
+		if (bts->nch.first_block == 0 || bts->nch.first_block == 1)
+			return true;
+		break;
+	case 2:
+		if (bts->nch.first_block == 0)
+			return true;
+		break;
+	default:
+		break;
+	}
+
+	/* anything else is not permitted */
+	return false;
+}
+
 static void bootstrap_bts(struct gsm_bts *bts)
 {
 	unsigned int n = 0;
@@ -393,6 +416,11 @@ static void bootstrap_bts(struct gsm_bts *bts)
 			     bts->si_common.chan_desc.bs_ag_blks_res);
 			bts->si_common.chan_desc.bs_ag_blks_res = 2;
 		}
+
+		if (!nch_position_compatible_with_combined_ccch(bts)) {
+			LOG_BTS(bts, DNM, LOGL_ERROR, "CCCH is combined with SDCCHs, but NCH position/size is "
+				"incompatible with that. Please fix your config!\n");
+		}
 	} else { /* Non-combined TS0/C0 configuration */
 		/* There can be additional CCCHs on even timeslot numbers */
 		n += (bts->c0->ts[2].pchan_from_config == GSM_PCHAN_CCCH);
@@ -401,6 +429,15 @@ static void bootstrap_bts(struct gsm_bts *bts)
 		bts->si_common.chan_desc.ccch_conf = (n << 1);
 	}
 
+	if (bts->nch.first_block + bts->nch.num_blocks > bts->si_common.chan_desc.bs_ag_blks_res) {
+		LOG_BTS(bts, DNM, LOGL_ERROR, "Position/Number of NCH blocks (%u..%u) exceeds AGCH (%u)."
+			"Please fix your config!\n", bts->nch.first_block,
+			bts->nch.first_block + bts->nch.num_blocks - 1,
+			bts->si_common.chan_desc.bs_ag_blks_res);
+	}
+
+	bts_setup_ramp_init_bts(bts);
+
 	/* ACC ramping is initialized from vty/config */
 
 	/* Initialize the BTS state */
@@ -453,7 +490,7 @@ static int inp_sig_cb(unsigned int subsys, unsigned int signal,
 			rate_ctr_inc(rate_ctr_group_get_ctr(trx->bts->bts_ctrs, BTS_CTR_BTS_OML_FAIL));
 			/* ip.access BTS models have a single global A-bis/OML link for all
 			 * transceivers, so once it's lost we need to notify them all. */
-			if (is_ipaccess_bts(trx->bts))
+			if (is_ipa_abisip_bts(trx->bts))
 				gsm_bts_all_ts_dispatch(trx->bts, TS_EV_OML_DOWN, NULL);
 			else /* Other BTS models (e.g. Ericsson) have per-TRX OML links */
 				gsm_trx_all_ts_dispatch(trx, TS_EV_OML_DOWN, NULL);
@@ -481,13 +518,12 @@ static int bsc_network_configure(const char *config_file)
 
 	rc = vty_read_config_file(config_file, NULL);
 	if (rc < 0) {
-		LOGP(DNM, LOGL_FATAL, "Failed to parse the config file: '%s'\n", config_file);
+		LOGP(DNM, LOGL_FATAL, "Failed to parse the config file: '%s' (%s)\n", config_file, strerror(-rc));
 		return rc;
 	}
 
 	/* start telnet after reading config for vty_get_bind_addr() */
-	rc = telnet_init_dynif(tall_bsc_ctx, bsc_gsmnet, vty_get_bind_addr(),
-			       OSMO_VTY_PORT_NITB_BSC);
+	rc = telnet_init_default(tall_bsc_ctx, bsc_gsmnet, OSMO_VTY_PORT_NITB_BSC);
 	if (rc < 0)
 		return rc;
 
@@ -518,6 +554,11 @@ static int bsc_vty_go_parent(struct vty *vty)
 		vty->node = CONFIG_NODE;
 		vty->index = NULL;
 		break;
+	case MGW_NODE:
+		vty->node = GSMNET_NODE;
+		vty->index = bsc_gsmnet;
+		vty->index_sub = NULL;
+		break;
 	case BTS_NODE:
 		vty->node = GSMNET_NODE;
 		{
@@ -614,7 +655,9 @@ static struct vty_app_info vty_info = {
 	.copyright	=
 	"Copyright (C) 2008-2018 Harald Welte, Holger Freyther\r\n"
 	"Contributions by Daniel Willmann, Jan Lübbe, Stefan Schmidt\r\n"
-	"Dieter Spaar, Andreas Eversberg, Sylvain Munaut, Neels Hofmeyr\r\n\r\n"
+	"Dieter Spaar, Andreas Eversberg, Sylvain Munaut, Neels Hofmeyr\r\n"
+	"Copyright (C) 2013-2022 sysmocom - s.f.m.c. GmbH\r\n"
+	"\r\n"
 	"License AGPLv3+: GNU AGPL version 3 or later <http://gnu.org/licenses/agpl-3.0.html>\r\n"
 	"This is free software: you are free to change and redistribute it.\r\n"
 	"There is NO WARRANTY, to the extent permitted by law.\r\n",
@@ -787,6 +830,12 @@ static const struct log_info_cat osmo_bsc_categories[] = {
 		.description = "Location Services",
 		.enabled = 1, .loglevel = LOGL_NOTICE,
 	},
+	[DASCI] = {
+		.name = "DASCI",
+		.description = "Advanced Speech Call Items (VGCS/VBS)",
+		.color = "\033[1;38m",
+		.enabled = 1, .loglevel = LOGL_NOTICE,
+	},
 	[DRESET] = {
 		.name = "DRESET",
 		.description = "RESET/ACK on A and Lb interfaces",
@@ -822,18 +871,21 @@ const struct log_info log_info = {
 extern void *tall_paging_ctx;
 extern void *tall_fle_ctx;
 extern void *tall_tqe_ctx;
-extern void *tall_ctr_ctx;
 
 static int bsc_mgw_setup(void)
 {
 	struct mgcp_client *mgcp_client_single;
 	unsigned int pool_members_initalized;
 
-	/* Initalize MGW pool. This initalizes and connects all MGCP clients that are currently configured in
+	/* Initialize MGW pool. This initializes and connects all MGCP clients that are currently configured in
 	 * the pool. Adding additional MGCP clients to the pool is possible but the user has to configure and
 	 * (re)connect them manually from the VTY. */
-	pool_members_initalized = mgcp_client_pool_connect(bsc_gsmnet->mgw.mgw_pool);
-	if (pool_members_initalized) {
+	if (!mgcp_client_pool_empty(bsc_gsmnet->mgw.mgw_pool)) {
+		pool_members_initalized = mgcp_client_pool_connect(bsc_gsmnet->mgw.mgw_pool);
+		if (!pool_members_initalized) {
+			LOGP(DNM, LOGL_ERROR, "MGW pool failed to initialize any pool members\n");
+			return -EINVAL;
+		}
 		LOGP(DNM, LOGL_NOTICE,
 		     "MGW pool with %u pool members configured, (ignoring MGW configuration in VTY node 'msc').\n",
 		     pool_members_initalized);
@@ -843,9 +895,9 @@ static int bsc_mgw_setup(void)
 	/* Initialize and connect a single MGCP client. This MGCP client will appear as the one and only pool
 	 * member if there is no MGW pool configured. */
 	LOGP(DNM, LOGL_NOTICE, "No MGW pool configured, using MGW configuration in VTY node 'msc'\n");
-        mgcp_client_single = mgcp_client_init(bsc_gsmnet, bsc_gsmnet->mgw.conf);
+	mgcp_client_single = mgcp_client_init(bsc_gsmnet, bsc_gsmnet->mgw.conf);
 	if (!mgcp_client_single) {
-		LOGP(DNM, LOGL_ERROR, "MGW (single) client initalization failed\n");
+		LOGP(DNM, LOGL_ERROR, "MGW (single) client initialization failed\n");
 		return -EINVAL;
 	}
 	if (mgcp_client_connect(mgcp_client_single)) {
@@ -873,7 +925,6 @@ int main(int argc, char **argv)
 	tall_paging_ctx = talloc_named_const(tall_bsc_ctx, 0, "paging_request");
 	tall_fle_ctx = talloc_named_const(tall_bsc_ctx, 0, "bs11_file_list_entry");
 	tall_tqe_ctx = talloc_named_const(tall_bsc_ctx, 0, "subch_txq_entry");
-	tall_ctr_ctx = talloc_named_const(tall_bsc_ctx, 0, "counter");
 
 	osmo_init_logging2(tall_bsc_ctx, &log_info);
 	osmo_stats_init(tall_bsc_ctx);
@@ -889,9 +940,8 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	bsc_gsmnet->mgw.conf = talloc_zero(bsc_gsmnet, struct mgcp_client_conf);
+	bsc_gsmnet->mgw.conf = mgcp_client_conf_alloc(bsc_gsmnet);
 	bsc_gsmnet->mgw.mgw_pool = mgcp_client_pool_alloc(bsc_gsmnet);
-	mgcp_client_conf_init(bsc_gsmnet->mgw.conf);
 
 	bts_init();
 	libosmo_abis_init(tall_bsc_ctx);
@@ -918,14 +968,11 @@ int main(int argc, char **argv)
 	/* seed the PRNG */
 	srand(time(NULL));
 
-	lchan_fsm_init();
-	bsc_subscr_conn_fsm_init();
-	assignment_fsm_init();
-	handover_fsm_init();
 	lb_init();
 	acc_ramp_global_init();
 	paging_global_init();
 	smscb_global_init();
+	meas_feed_txqueue_max_length_set(MEAS_FEED_TXQUEUE_MAX_LEN_DEFAULT);
 
 	/* Read the config */
 	rc = bsc_network_configure(config_file);
@@ -941,9 +988,7 @@ int main(int argc, char **argv)
 
 	/* start control interface after reading config for
 	 * ctrl_vty_get_bind_addr() */
-	bsc_gsmnet->ctrl = bsc_controlif_setup(bsc_gsmnet,
-					       ctrl_vty_get_bind_addr(),
-					       OSMO_CTRL_PORT_NITB_BSC);
+	bsc_gsmnet->ctrl = bsc_controlif_setup(bsc_gsmnet, OSMO_CTRL_PORT_NITB_BSC);
 	if (!bsc_gsmnet->ctrl) {
 		fprintf(stderr, "Failed to init the control interface. Exiting.\n");
 		exit(1);
@@ -989,6 +1034,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (bsc_mgw_setup() != 0)
+		exit(1);
+
 	llist_for_each_entry(msc, &bsc_gsmnet->mscs, entry) {
 		if (osmo_bsc_msc_init(msc) != 0) {
 			LOGP(DMSC, LOGL_ERROR, "Failed to start up. Exiting.\n");
@@ -996,9 +1044,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (bsc_mgw_setup() != 0)
-		exit(1);
-
 	if (osmo_bsc_sigtran_init(&bsc_gsmnet->mscs) != 0) {
 		LOGP(DNM, LOGL_ERROR, "Failed to initialize sigtran backhaul.\n");
 		exit(1);
